@@ -7,12 +7,16 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include <cassert>
+#include <set>
 #include <sstream>
 #include <string>
 #include <algorithm>
 #include <string.h>
 #include <librevenge-stream/librevenge-stream.h>
 #include <zlib.h>
+
+#include "MSPUBMetaData.h"
 #include "MSPUBParser.h"
 #include "MSPUBCollector.h"
 #include "MSPUBBlockID.h"
@@ -114,6 +118,11 @@ bool MSPUBParser::parse()
   MSPUB_DEBUG_MSG(("***NOTE***: Where applicable, the meanings of block/chunk IDs and Types printed below may be found in:\n\t***MSPUBBlockType.h\n\t***MSPUBBlockID.h\n\t***MSPUBContentChunkType.h\n*****\n"));
   if (!m_input->isStructured())
     return false;
+  librevenge::RVNGInputStream *metaData = m_input->getSubStreamByName("\x05SummaryInformation");
+  if (metaData)
+    // No check: metadata are not important enough to fail if they can't be parsed
+    parseMetaData(metaData);
+  delete metaData;
   librevenge::RVNGInputStream *quill = m_input->getSubStreamByName("Quill/QuillSub/CONTENTS");
   if (!quill)
   {
@@ -475,6 +484,7 @@ bool MSPUBParser::parseFontChunk(
         {
           boost::optional<librevenge::RVNGString> name;
           boost::optional<unsigned> eotOffset;
+          unsigned eotLength = 0;
           input->seek(subInfo.dataOffset + 4, librevenge::RVNG_SEEK_SET);
           while (stillReading(input, subInfo.dataOffset + subInfo.dataLength))
           {
@@ -482,19 +492,29 @@ bool MSPUBParser::parseFontChunk(
             if (subSubInfo.id == EMBEDDED_FONT_NAME)
             {
               name = librevenge::RVNGString();
-              appendCharacters(name.get(), subSubInfo.stringData, "UTF-16");
+              // drop trailing 0
+              // TODO: This could be a general problem. Check.
+              const std::size_t len = subSubInfo.stringData.size();
+              if ((len > 2) && (subSubInfo.stringData[len - 1] == 0) && (subSubInfo.stringData[len - 2] == 0))
+              {
+                subSubInfo.stringData.pop_back();
+                subSubInfo.stringData.pop_back();
+              }
+              appendCharacters(name.get(), subSubInfo.stringData, "UTF-16LE");
             }
             else if (subSubInfo.id == EMBEDDED_EOT)
             {
               eotOffset = subSubInfo.dataOffset;
+              eotLength = subSubInfo.dataLength;
             }
           }
-          if (!!name && !!eotOffset)
+          if (bool(name) && bool(eotOffset))
           {
-            input->seek(eotOffset.get(), librevenge::RVNG_SEEK_SET);
-            MSPUBBlockInfo eotRecord = parseBlock(input, true);
-            librevenge::RVNGBinaryData &data = m_collector->addEOTFont(name.get());
-            unsigned long toRead = eotRecord.dataLength;
+            // skip length, we've already read that
+            // TODO: Why do we not read the data as part of the block info?
+            input->seek(eotOffset.get() + 4, librevenge::RVNG_SEEK_SET);
+            librevenge::RVNGBinaryData data;
+            unsigned long toRead = eotLength;
             while (toRead > 0 && stillReading(input, (unsigned long)-1))
             {
               unsigned long howManyRead = 0;
@@ -502,6 +522,7 @@ bool MSPUBParser::parseFontChunk(
               data.append(buf, howManyRead);
               toRead -= howManyRead;
             }
+            m_collector->addEOTFont(name.get(), data);
             input->seek(subInfo.dataOffset + subInfo.dataLength, librevenge::RVNG_SEEK_SET);
           }
         }
@@ -647,6 +668,7 @@ bool MSPUBParser::parseShape(librevenge::RVNGInputStream *input,
     boost::optional<unsigned> numRows;
     boost::optional<unsigned> numCols;
     boost::optional<unsigned> rowcolArrayOffset;
+    boost::optional<unsigned> textId;
     while (stillReading(input, pos + length))
     {
       MSPUBBlockInfo info = parseBlock(input, true);
@@ -674,8 +696,12 @@ bool MSPUBParser::parseShape(librevenge::RVNGInputStream *input,
       {
         rowcolArrayOffset = info.dataOffset;
       }
+      else if (info.id == SHAPE_TEXT_ID)
+      {
+        textId = info.data;
+      }
     }
-    if (!!cellsSeqNum && !!numRows && !!numCols && !!rowcolArrayOffset)
+    if (bool(cellsSeqNum) && bool(numRows) && bool(numCols) && bool(rowcolArrayOffset))
     {
       unsigned nr = numRows.get();
       unsigned nc = numCols.get();
@@ -733,6 +759,11 @@ bool MSPUBParser::parseShape(librevenge::RVNGInputStream *input,
           break;
         }
       }
+
+      TableInfo ti(nr, nc);
+      ti.m_rowOffsetsInEmu = rowOffsetsInEmu;
+      ti.m_columnOffsetsInEmu = columnOffsetsInEmu;
+
       if (!index)
       {
         MSPUB_DEBUG_MSG(("WARNING: Couldn't find cells of seqnum %u corresponding to table of seqnum %u.\n",
@@ -741,12 +772,70 @@ bool MSPUBParser::parseShape(librevenge::RVNGInputStream *input,
       }
       else
       {
-        // Currently do nothing with the cells chunk.
+        const ContentChunkReference &cellsChunk = m_contentChunks[m_cellsChunkIndices[get(index)]];
+        input->seek(cellsChunk.offset, librevenge::RVNG_SEEK_SET);
+        const unsigned cellsLength = readU32(input);
+        boost::optional<unsigned> cellCount;
+        while (stillReading(input, cellsChunk.offset + cellsLength))
+        {
+          MSPUBBlockInfo info = parseBlock(input, true);
+          switch (info.id)
+          {
+          case 0x01:
+            cellCount = info.data;
+            break;
+          case 0x02:
+          {
+            input->seek(info.dataOffset + 4, librevenge::RVNG_SEEK_SET);
+            while (stillReading(input, info.dataOffset + info.dataLength))
+            {
+              const MSPUBBlockInfo itemInfo = parseBlock(input, true);
+              if (itemInfo.id == 0)
+              {
+                input->seek(itemInfo.dataOffset + 4, librevenge::RVNG_SEEK_SET);
+                CellInfo currentCell;
+                while (stillReading(input, itemInfo.dataOffset + itemInfo.dataLength))
+                {
+                  const MSPUBBlockInfo subInfo = parseBlock(input, true);
+                  switch (subInfo.id)
+                  {
+                  case 0x01:
+                    currentCell.m_startRow = subInfo.data;
+                    break;
+                  case 0x02:
+                    currentCell.m_endRow = subInfo.data;
+                    break;
+                  case 0x03:
+                    currentCell.m_startColumn = subInfo.data;
+                    break;
+                  case 0x04:
+                    currentCell.m_endColumn = subInfo.data;
+                    break;
+                  // TODO: 0x09 - 0x0e: width/height of content + margins?
+                  default:
+                    break;
+                  }
+                }
+                ti.m_cells.push_back(currentCell);
+              }
+            }
+
+            break;
+          }
+          default:
+            break;
+          }
+        }
+
+        if (bool(cellCount) && (get(cellCount) != ti.m_cells.size()))
+        {
+          MSPUB_DEBUG_MSG(("%u cell records expected, but read %u\n", get(cellCount), unsigned(ti.m_cells.size())));
+        }
       }
-      TableInfo ti(nr, nc);
-      ti.m_rowOffsetsInEmu = rowOffsetsInEmu;
-      ti.m_columnOffsetsInEmu = columnOffsetsInEmu;
+
       m_collector->setShapeTableInfo(chunk.seqNum, ti);
+      if (bool(textId))
+        m_collector->addTextShape(get(textId), chunk.seqNum);
       return true;
     }
     return false;
@@ -862,11 +951,18 @@ bool MSPUBParser::parseQuill(librevenge::RVNGInputStream *input)
   MSPUB_DEBUG_MSG(("MSPUBParser::parseQuill\n"));
   unsigned chunkReferenceListOffset = 0x18;
   std::list<QuillChunkReference> chunkReferences;
+  std::set<unsigned> readChunks; // guard against cycle in the chunk list
   while (chunkReferenceListOffset != 0xffffffff)
   {
     input->seek(chunkReferenceListOffset + 2, librevenge::RVNG_SEEK_SET);
     unsigned short numChunks = readU16(input);
     chunkReferenceListOffset = readU32(input);
+    if (readChunks.find(chunkReferenceListOffset) != readChunks.end())
+    {
+      MSPUB_DEBUG_MSG(("Found a cycle in chunk reference list: a broken file!\n"));
+      break;
+    }
+    readChunks.insert(chunkReferenceListOffset);
     for (unsigned i = 0; i < numChunks; ++i)
     {
       QuillChunkReference quillChunkReference = parseQuillChunkReference(input);
@@ -885,6 +981,7 @@ bool MSPUBParser::parseQuill(librevenge::RVNGInputStream *input)
   std::vector<unsigned> textLengths;
   std::vector<unsigned> textIDs;
   std::vector<unsigned> textOffsets;
+  std::map<unsigned, std::vector<unsigned> > tableCellTextEnds;
   unsigned textOffsetAccum = 0;
   std::vector<TextSpanReference> spans;
   std::vector<TextParagraphReference> paras;
@@ -957,8 +1054,7 @@ bool MSPUBParser::parseQuill(librevenge::RVNGInputStream *input)
     else if (i->name == "TCD ")
     {
       input->seek(i->offset, librevenge::RVNG_SEEK_SET);
-      std::vector<unsigned> ends = parseTableCellDefinitions(input, *i);
-      m_collector->setNextTableCellTextEnds(ends);
+      tableCellTextEnds[i->id] = parseTableCellDefinitions(input, *i);
     }
   }
   if (parsedStrs && parsedSyid && parsedFdpc && parsedFdpp && parsedStsh && parsedFont && textChunkReference != chunkReferences.end())
@@ -973,7 +1069,7 @@ bool MSPUBParser::parseQuill(librevenge::RVNGInputStream *input)
       std::vector<TextParagraph> readParas;
       std::vector<TextSpan> readSpans;
       std::vector<unsigned char> text;
-      for (unsigned k = 0; k < textLengths[j]; ++k)
+      for (unsigned k = 0; k < textLengths[j] && currentTextPara != paras.end() && currentTextSpan != spans.end(); ++k)
       {
         text.push_back(readU8(input));
         text.push_back(readU8(input));
@@ -1005,19 +1101,22 @@ bool MSPUBParser::parseQuill(librevenge::RVNGInputStream *input)
           readSpans.clear();
         }
       }
-      if (!readSpans.empty())
+      if (!text.empty() && currentTextSpan != spans.end())
+      {
+        readSpans.push_back(TextSpan(text, currentTextSpan->charStyle));
+        MSPUB_DEBUG_MSG(("Saw text span %d in the current text paragraph.\n", (unsigned)readSpans.size()));
+      }
+      text.clear();
+      if (!readSpans.empty() && currentTextPara != paras.end())
       {
-        if (!text.empty())
-        {
-          readSpans.push_back(TextSpan(text, currentTextSpan->charStyle));
-          MSPUB_DEBUG_MSG(("Saw text span %d in the current text paragraph.\n", (unsigned)readSpans.size()));
-        }
-        text.clear();
         readParas.push_back(TextParagraph(readSpans, currentTextPara->paraStyle));
         MSPUB_DEBUG_MSG(("Saw paragraph %d in the current text block.\n", (unsigned)readParas.size()));
       }
       m_collector->addTextString(readParas, textIDs[j]);
       m_collector->setTextStringOffset(textIDs[j], textOffsets[j]);
+      const std::map<unsigned, std::vector<unsigned> >::const_iterator it = tableCellTextEnds.find(j);
+      if (it != tableCellTextEnds.end())
+        m_collector->setTableCellTextEnds(textIDs[j], it->second);
     }
     textChunkReference = chunkReferences.end();
   }
@@ -1504,7 +1603,7 @@ void MSPUBParser::parseEscherShape(librevenge::RVNGInputStream *input, const Esc
         {
           maybe_tertiaryFoptValues = extractEscherValues(input, cTertiaryFopt);
         }
-        if (!!maybe_tertiaryFoptValues)
+        if (bool(maybe_tertiaryFoptValues))
         {
           const std::map<unsigned short, unsigned> &tertiaryFoptValues =
             maybe_tertiaryFoptValues.get();
@@ -1570,7 +1669,7 @@ void MSPUBParser::parseEscherShape(librevenge::RVNGInputStream *input, const Esc
             }
             else
             {
-              if (!!maybe_tertiaryFoptValues)
+              if (bool(maybe_tertiaryFoptValues))
               {
                 std::map<unsigned short, unsigned> &tertiaryFoptValues =
                   maybe_tertiaryFoptValues.get();
@@ -1687,7 +1786,7 @@ void MSPUBParser::parseEscherShape(librevenge::RVNGInputStream *input, const Esc
                                         dotStyle));
           }
 
-          if (!!maybe_tertiaryFoptValues)
+          if (bool(maybe_tertiaryFoptValues))
           {
             std::map<unsigned short, unsigned> &tertiaryFoptValues = maybe_tertiaryFoptValues.get();
             unsigned *ptr_numColumns = getIfExists(tertiaryFoptValues, FIELDID_NUM_COLUMNS);
@@ -2212,6 +2311,12 @@ std::map<unsigned short, unsigned> MSPUBParser::extractEscherValues(librevenge::
   while (stillReading(input, record.contentsOffset + record.contentsLength))
   {
     unsigned short id = readU16(input);
+    if (id == 0)
+    {
+      if (!stillReading(input, record.contentsOffset + record.contentsLength))
+        break;
+      MSPUB_DEBUG_MSG(("found escher value with ID 0!\n"));
+    }
     unsigned value = readU32(input);
     ret[id] = value;
   }
@@ -2434,6 +2539,20 @@ void MSPUBParser::parsePaletteEntry(librevenge::RVNGInputStream *input, MSPUBBlo
   }
 }
 
+bool MSPUBParser::parseMetaData(librevenge::RVNGInputStream *const input)
+{
+  assert(input);
+
+  MSPUBMetaData metaData;
+  metaData.parse(input);
+  m_input->seek(0, librevenge::RVNG_SEEK_SET);
+  metaData.parseTimes(m_input);
+  m_collector->collectMetaData(metaData.getMetaData());
+
+  return true;
+}
+
+
 }
 
 /* vim:set shiftwidth=2 softtabstop=2 expandtab: */
